<?php



// Restrict category selection for users

add_filter('map_meta_cap', 'crr_restrict_categories', 10, 4);



// Restrict categories based on user permissions

function crr_restrict_categories($caps, $cap, $user_id, $args) {

    if ($cap === 'edit_post' && isset($args[0])) {

        $post_id = $args[0];

        $post = get_post($post_id);



        if ($post && $post->post_type === 'post') {

            $user_restrictions = get_option('crr_user_restrictions', array());



            if (isset($user_restrictions[$user_id])) {

                $allowed_categories = $user_restrictions[$user_id];

                $post_categories = wp_get_post_categories($post_id);



                foreach ($post_categories as $cat_id) {

                    if (!in_array($cat_id, $allowed_categories)) {

                        $caps[] = 'do_not_allow';

                        break;

                    }

                }

            }

        }

    }



    return $caps;

}



add_action('admin_head-post-new.php', 'crr_restrict_category_ui');

add_action('admin_head-post.php', 'crr_restrict_category_ui');



// Restrict category UI based on user permissions

function crr_restrict_category_ui() {

    $user_id = get_current_user_id();

    $user_restrictions = get_option('crr_user_restrictions', array());



    if (!isset($user_restrictions[$user_id])) {

        return;

    }



    $allowed_categories = $user_restrictions[$user_id];

    $all_categories = get_categories(array('hide_empty' => false));

    $disallowed_categories = array_diff(wp_list_pluck($all_categories, 'term_id'), $allowed_categories);



    if (!empty($disallowed_categories)) {

        echo '<style>';

        foreach ($disallowed_categories as $cat_id) {

            echo '.categorychecklist li[id$="-' . $cat_id . '"] { display: none; }';

        }

        echo '</style>';

    }

}